
// 
// Test program for configuring and using the Mini USB DMX interface as a DMX transmitter.
//
// DMX channel value (same for all channels) can be given on the command line.
// The program will send out DMX frames until a key is pressed.
//
// Author: Mats Byggmastar
// Date:   13.07.2006
//

#include <stdio.h>
#include <conio.h>
#include <windows.h>
#include "FTD2XX.H"


int main(int argc, char * argv[])
{
    unsigned char ucChannelValue = 255;
    unsigned i;
    unsigned uFrames = 0;
    DWORD dwDevices;
    FT_HANDLE ftHandle;
    FTDCB ftDCB;
    char acBuf[64];
    unsigned char aucDmxData[1 + 512];
    ULONG uBytesWritten;

    // Get number of devices
    dwDevices = 0;
    if (FT_ListDevices(&dwDevices, 0, FT_LIST_NUMBER_ONLY) != FT_OK)
    {
        printf("Can't determine number of devices!\n");
        return -1;
    }
    printf("Number of devices: %u\n", dwDevices);
    if (dwDevices == 0)
    {
        printf("No devices!\n");
        return 0;
    }

    // List devices
    for (i=0; i < dwDevices; i++)
    {
        if (FT_ListDevices((PVOID)i, acBuf, FT_LIST_BY_INDEX | FT_OPEN_BY_DESCRIPTION) != FT_OK)
        {
            printf("Can't get device name!\n");
            return -1;
        }
        printf("Device %u: %s\n", i, acBuf);
    }

    // Open the first device
    FT_ListDevices(0, acBuf, FT_LIST_BY_INDEX | FT_OPEN_BY_DESCRIPTION);
    ftHandle = FT_W32_CreateFile(acBuf, GENERIC_READ|GENERIC_WRITE, 0, 0,
                                 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FT_OPEN_BY_DESCRIPTION, 0);
    if (ftHandle == INVALID_HANDLE_VALUE) 
    {
        printf("Unable to open device 0: %s\n", acBuf);
        return -1;
    }

    // Get current DCB
    if (!FT_W32_GetCommState(ftHandle, &ftDCB)) 
    {
        printf("Can't get CommState!\n");
        return -1;
    }

    // Set baud rate and line coding
    ftDCB.BaudRate = 250000;
    ftDCB.Parity = FT_PARITY_NONE;
    ftDCB.StopBits = FT_STOP_BITS_2;
    ftDCB.ByteSize = FT_BITS_8;
    ftDCB.fOutX = FALSE;
    ftDCB.fInX = FALSE;
    ftDCB.fErrorChar = FALSE;
    ftDCB.fBinary = TRUE;
    ftDCB.fRtsControl = FALSE;
    ftDCB.fAbortOnError = FALSE;
    if (!FT_W32_SetCommState(ftHandle, &ftDCB)) 
    {
        printf("Unable to set baud rate and line coding!\n");
        return -1;
    }

    // Purge device
    if (!FT_W32_PurgeComm(ftHandle, FT_PURGE_TX | FT_PURGE_RX))
    {
        printf("PurgeComm failed!\n");
        return -1;
    }

    // Clear RTS so DMX interface will operate as transmitter
    if (!FT_W32_EscapeCommFunction(ftHandle, CLRRTS))
    {
        printf("EscapeCommFunction failed!\n");
        return -1;
    }

    // Get DMX channel value from command line if present
    if (argc > 1)
    {
        ucChannelValue = atoi(argv[1]);
    }

    // Set DMX start code: normal dimmer/level data
    aucDmxData[0] = 0;

    // Set DMX channel values
    printf("Set DMX channel values to: %u\n", (unsigned)ucChannelValue);
    for (i=1; i < sizeof(aucDmxData); i++)
    {
        aucDmxData[i] = ucChannelValue;
    }

    while (!kbhit())
    {
        printf("\rSending DMX frame #%u ", uFrames);

        // Signal start of new DMX frame
        if (!FT_W32_SetCommBreak(ftHandle))
        {
            printf("SetCommBreak failed!\n");
            return -1;
        }
        if (!FT_W32_ClearCommBreak(ftHandle))
        {
            printf("ClearCommBreak failed!\n");
            return -1;
        }

        // Send DMX frame
        uBytesWritten = 0;
        if (!FT_W32_WriteFile(ftHandle, aucDmxData, sizeof(aucDmxData), &uBytesWritten, NULL))
        {
            printf("WriteFile failed!\n");
            return -1;
        }
        if (uBytesWritten != sizeof(aucDmxData))
        {
            printf("Only %u bytes written!\n", uBytesWritten);
        }

        uFrames++;
    }

    // Set RTS so DMX the interface will stop sending
    if (!FT_W32_EscapeCommFunction(ftHandle, SETRTS))
    {
        printf("EscapeCommFunction failed!\n");
        return -1;
    }

    // Close device
    FT_W32_CloseHandle(ftHandle);

    return 0;
}

